<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:xs="http://www.w3.org/2001/XMLSchema"
    xmlns:math="http://www.w3.org/2005/xpath-functions/math" xmlns:map="http://www.w3.org/2005/xpath-functions/map" xmlns:doc="http://www.saxonica.com/ns/doc"
    xmlns:f="MyFunctions" xmlns:svg="http://www.w3.org/2000/svg" xmlns:saxon="http://saxon.sf.net/" exclude-result-prefixes="xs math f doc" version="3.0">

    <doc:synopsis title="Template time display">
        <p>Generate a set of displays for template times</p>
    </doc:synopsis>

    <!-- *IMPORTANT* This needs the '-l' option on Saxon to track source line numbers -->

    <xsl:mode name="stats.time" on-no-match="shallow-copy"/>
    <xsl:template match="stats" mode="stats.time">
        <xsl:apply-templates mode="#current"/>
    </xsl:template>
    <xsl:template match="group" mode="stats.time"/>

    <xsl:template match="group[tokenize(@show, '\s+') = 'time']" mode="stats.time">
        <xsl:apply-templates select="." mode="stats.barchart">
            <xsl:with-param name="name" tunnel="yes">times</xsl:with-param>
            <xsl:with-param name="valueOfInterest" as="function(*)"
                select="
                    function ($c) {
                        $c/@times.total
                    }" tunnel="yes"/>
            <xsl:with-param name="options" as="map(*)" tunnel="yes"
                select="map{
                'axis.x.format' : function($a) {format-number($a idiv 1000000,'0')} ,
                'axis.x.label' : 'Total / ms', 
                'label' : 'Template pattern match time'}"
            />
        </xsl:apply-templates>
    </xsl:template>
    <xsl:mode name="stats.time.svg" on-no-match="shallow-skip"/>
    <xsl:template match="group" mode="stats.time.svg"/>
    <xsl:template match="group[tokenize(@show, '\s+') = 'time']" mode="stats.time.svg" as="element()*">
        <xsl:variable name="data.format" as="function(*)"
            select="
                function ($a) {
                    format-number($a idiv 1000000, ',##0')
                }"
            doc:doc="Results in ns - display in ms"/>
        <xsl:apply-templates select="." mode="stats.barchart.svg">
            <xsl:with-param name="name" tunnel="yes">times</xsl:with-param>
            <xsl:with-param name="valueOfInterest" as="function(*)"
                select="
                    function ($c) {
                        $c/@times.total
                    }" tunnel="yes"/>
            <xsl:with-param name="options" as="map(*)" tunnel="yes"
                select="map{
                'axis.x.format' : $data.format,
                'axis.x.label' : 'Total / ms', 
                'label' : 'Template pattern match time'}"/>
            <xsl:with-param name="data.format" as="function(*)" select="$data.format" tunnel="yes"/>
        </xsl:apply-templates>
    </xsl:template>
    <!--<xsl:variable name="stylesheet.templates"
            select="map:merge($sources ! map:entry(tokenize(., '/')[last()], doc(.)//xsl:template[@match]))"/>
        <xsl:variable name="sources.short"
            select="distinct-values($sources ! tokenize(., '/')[last()])"/>
        <xsl:variable name="short2long"
            select="map:merge($sources ! map:entry(tokenize(., '/')[last()], .))"/>
            <xsl:variable name="sources.matches"
            select="map:merge($sources ! map:entry(., map:merge(doc(.)//xsl:template/@match/map:entry(saxon:line-number(.), .))))"/>-->

    <xsl:mode name="stats.time.table" on-no-match="shallow-skip"/>
    <xsl:template match="group" mode="stats.time.table"/>
    <xsl:template match="group[tokenize(@show, '\s+') = 'time']" mode="stats.time.table" as="element()*">
        <xsl:variable name="data.format" as="function(*)"
            select="
                function ($a) {
                    format-number($a idiv 1000000, '0')
                }"
            doc:doc="Results in ns - display in ms"/>
        <div name="times">
            <p> Only showing significant matches</p>
            <xsl:apply-templates select="." mode="stats.table">
                <xsl:with-param name="name" tunnel="yes">times</xsl:with-param>
                <xsl:with-param name="valueOfInterest" as="function(*)"
                    select="
                        function ($c) {
                            $c/@times.total
                        }" tunnel="yes"/>
               <xsl:with-param name="data.format" as="function(*)" select="$data.format" tunnel="yes"/>
            </xsl:apply-templates>
        </div>
    </xsl:template>

</xsl:stylesheet>
